@@ -47,9 +47,9 @@ data PubKey =
 instance ASN1Object PubKey where
     fromASN1 (Start Sequence:Start Sequence:OID pkalg:xs)
         | pkalg == getObjectID PubKeyALG_RSA =
-            case xs of
-                Null:End Sequence:BitString bits:End Sequence:xs2 -> decodeASN1Err "RSA" bits xs2 (toPubKeyRSA . fromASN1)
-                _ -> Left "fromASN1: X509.PubKey: unknown RSA format"
+            case removeNull xs of
+                End Sequence:BitString bits:End Sequence:xs2 -> decodeASN1Err "RSA" bits xs2 (toPubKeyRSA . fromASN1)
+                _ -> Left ("fromASN1: X509.PubKey: unknown RSA format: " ++ show xs)
         | pkalg == getObjectID PubKeyALG_DSA   =
             case xs of
                 Start Sequence:IntVal p:IntVal q:IntVal g:End Sequence:End Sequence:BitString bits:End Sequence:xs2 ->
@@ -60,23 +60,29 @@ instance ASN1Object PubKey where
                                                                                         , DSA.params_g = g
                                                                                         }
                                                                            , DSA.public_y = dsapub }
-                             in Right (PubKeyDSA pubkey, xs2)
+                             in Right (PubKeyDSA pubkey, [])
                         _ -> Left "fromASN1: X509.PubKey: unknown DSA format"
                         )
                 _ -> Left "fromASN1: X509.PubKey: unknown DSA format"
         | pkalg == getObjectID PubKeyALG_ECDSA =
             case xs of
-                OID [1,3,132,0,34]:End Sequence:BitString bits:End Sequence:xs2 -> Right (PubKeyECDSA ECC.SEC_p384r1 (bitArrayGetData bits), xs2)
+                OID curveOid:End Sequence:BitString bits:End Sequence:xs2 ->
+                    case fromObjectID curveOid of
+                        Just curveName -> Right (PubKeyECDSA curveName (bitArrayGetData bits), xs2)
+                        Nothing        -> Left ("fromASN1: X509.Pubkey: ECDSA unknown curve " ++ show curveOid)
                 _ -> Left "fromASN1: X509.PubKey: unknown ECDSA format"
-        | otherwise = undefined
-        where decodeASN1Err format bits xs2 f =
+        | otherwise = error ("unknown public key OID: " ++ show pkalg)
+      where decodeASN1Err format bits xs2 f =
                 case decodeASN1' BER (bitArrayGetData bits) of
                     Left err -> Left ("fromASN1: X509.PubKey " ++ format ++ " bitarray cannot be parsed: " ++ show err)
                     Right s  -> case f s of
                                     Left err -> Left err
                                     Right (r, xsinner) -> Right (r, xsinner ++ xs2)
-              toPubKeyRSA = either Left (\(rsaKey, r) -> Right (PubKeyRSA rsaKey, r))
-            
+            toPubKeyRSA = either Left (\(rsaKey, r) -> Right (PubKeyRSA rsaKey, r))
+
+            removeNull (Null:xs) = xs
+            removeNull l         = l
+
     fromASN1 l = Left ("fromASN1: X509.PubKey: unknown format:" ++ show l)
     toASN1 a = \xs -> encodePK a ++ xs
 
@@ -109,6 +115,6 @@ encodePK key = asn1Container Sequence (encodeInner key)
         eOid = case curveName of
                     ECC.SEC_p384r1 -> [1,3,132,0,34]
                     _              -> error ("undefined curve OID: " ++ show curveName)
-    encodeInner (PubKeyDH _) = undefined
+    encodeInner (PubKeyDH _) = error "encodeInner: unimplemented public key DH"
     encodeInner (PubKeyUnknown _ l) =
         asn1Container Sequence [pkalg,Null] ++ [BitString $ toBitArray l 0]
